#!/usr/bin/perl

use strict;
use Getopt::Std;
use nde;

my $dflt_tftp_server = "10.10.140.2" ; 
my $dflt_tftp_dir = "/tftpboot" ;

my %opts;
getopts( 'vqhns:d:f:p:u:', \%opts );
if ( $opts{ 'h' } ) { usage(); }

my $tftp_server = $opts{ 'f' };
if ( ! $tftp_server ) { $tftp_server = $dflt_tftp_server; }

my $tftp_dir = $opts{ 'd' };
if ( ! $tftp_dir ) { $tftp_dir = $dflt_tftp_dir; }

my $sub_dir = $opts{ 's' };
if ( $sub_dir )
{ 
	-d "$tftp_dir/$sub_dir" || system( "mkdir", "-p", "$tftp_dir/$sub_dir" );
	$sub_dir = $sub_dir . "/";
} 

my $postfix;
if ( $opts{ 'p' } ) { $postfix = "." . $opts{ 'p' }; } 


my $debug;
if( $opts{ 'v' } ) { $debug = "YES" ; }

if ( ! $ARGV[0] ) { usage(); }

my $username;
my $passwd;
if( $opts{ 'u' } )
{
	open( FILE, $opts{ 'u' } );
	my $line = <FILE>;
	chomp($line);
	($username, $passwd, $opts{ 'u' }) = split( " ", $line );
	close(FILE);
}
else
{
	$username = `/usr/bin/whoami`;
	chomp($username); # don't forget this!
	$passwd = nde::get_password();
}


foreach my $device (@ARGV)
{
	$opts{ 'q' } || print "fetching running config from $device\n";
	my $rh = nde::create_session( "", $device, $username, $passwd, $debug  );
	nde::login();
	nde::get_type();
	my $dest = "${sub_dir}${device}${postfix}";
	system( "touch", "$tftp_dir/$dest" );
	system( "chmod", "o+w", "$tftp_dir/$dest" );
	my $error = nde::tftp_from_configuration( $tftp_server, $dest );
	if( $error ) { print "$device ERROR:\n$error\n"; }
	system( "chmod", "o-w", "$tftp_dir/$dest" );
	nde::logout();
	$opts{ 'q' } || print "done with $device\n";
}
exit 0;


sub usage
{
	print "usage: $0 [-vh] [-f tftp_server] [-p postfix] [-d tftp_dir] [-s sub_dir] [-u file] device ...\n";
	print "connects to the listed device(s), and copies\n";
	print "device's running configuration to file named 'device' on tftp_server.\n";
	print "default tftp_server is $dflt_tftp_server\n";
	print "if tftp_server is the localhost, then file creation/permission setting\n";
	print "is handled by the program, otherwise you are on your own.\n";
	print "\n";
	print "options:\n";
	print "-h  =  print this message\n";
	print "-v  =  view interactive sessions as they happen\n";
	print "-q  =  quiet mode (only prints errors)\n";
	print "-p postfix = config is copied to file named 'device.postfix'\n";
	print "-f tftp_server = tftp_server is used instead of the default server\n";
	print "-d tftp_dir = root of tftp filesystem, default value is $dflt_tftp_dir\n";
	print "-s sub_dir = sub directory of tftp filesystem to copy config files to, default value is to put files in the root of tftp filesystem\n";
	print "-u file = loads username and password from file, expects a single line of format 'username password', scary but useful for scripted jobs\n";
	exit 1;
}

