#!/usr/bin/perl

use strict;
use Getopt::Std;
use nde;

my $dflt_tftp_server = "10.10.140.2" ; 

my $timeout;

my %opts;

getopts( 'vhstnf:d:u:p:w:', \%opts );
if ( $opts{ 'h' } ) { usage(); }

my $tftp_server = $opts{ 'f' };
if ( ! $tftp_server ) { $tftp_server = $dflt_tftp_server; }


my $src = shift(@ARGV);

my $debug;
if( $opts{ 'v' } ) { $debug = "YES" ; }
if( $opts{ 'w' } ) { $timeout = $opts{ 'w' }; }

if ( ! $ARGV[0] ) { usage(); }

my $username;
my $passwd;
if( $opts{ 'p' } )
{
	open( FILE, $opts{ 'p' } );
	my $line = <FILE>;
	chomp($line);
	($username, $passwd, $opts{ 'p' }) = split( " ", $line );
	close(FILE);
}
else
{
	$username = `/usr/bin/whoami`;
	chomp($username); # don't forget this!
	$passwd = nde::get_password();
}
if ( $opts{ 'u' } )
{
	$username = $opts{ 'u' };
}

foreach my $device (@ARGV)
{
	my $rh = nde::create_session( "", $device, $username, $passwd, $debug  );
	nde::login();
	nde::get_type();
	my $file = $src;
	if ( $opts{ 't' } ) { $file = $file . ".$$rh{ 'type' }"; }
	if ( $opts{ 'n' } ) { $file = $file . ".$device"; }
	print "copying $file to running config on $device\n";
	my $error = nde::tftp_to_configuration( $tftp_server, $file, $timeout );
	if( $error ) { print "$device ERROR:\n$error\n"; }
	if( (! $error) && $opts{ 's' } )
	{
		my $error = nde::save_configuration();
		if( $error ) { print "ERROR:\n$error\n"; }
	}
	nde::logout();
	print "done with $device\n";
}
exit 0;


sub usage
{
	print "usage: $0 [-vhstn] [-f tftp_server] [-w seconds] [-u username] [-p file] src device ...\n";
	print "connects to the listed device(s), and copies\n";
	print "src FROM tftp_server to the running config on device.\n";
	print "default tftp_server is $dflt_tftp_server\n";
	print "\n";
	print "options:\n";
	print "-h  =  print this message\n";
	print "-v  =  view interactive sessions as they happen\n";
	print "-s  =  save running configuration after tftping file\n";
	print "-t  =  type of device is appended to src\n";
	print "       for example.\n";
	print "       on an ios device, the file to copy becomes src.ios\n";
	print "       on a catos device, the file to copy becomes src.catos\n";
	print "       on an extreme device, the file to copy becomes src.extreme\n";
	print "       needless to say, these files should exist and reflect\n";
	print "       proper configuration syntax for the appropriate device type\n";
	print "-n  =  name of device is appended to src\n";
	print "       so src becomes src.device\n";
	print "       useful for pushing device specific configs\n";
	print "-f tftp_server = tftp_server is used instead of the default server\n";
	print "-u username = use username when logging onto devices, by default uses output of whoami command as username\n";
	print "-p file = loads username and password from file, expects a single line of format 'username password', scary but useful for scripted jobs\n";
	print "-w seconds = how long to wait for a response back from device when tftping, default value is 60\n";

	exit 1;
}

