# Common OIDs Query Module 
# A perl module for querying some common SNMP OIDs
#
# written by Scott Matott sXe (matott@mcs.anl.gov)
#	     Scott Behrens (behrens@mcs.anl.gov)		

package nde;

$VERSION = '0.0.1';  # alpha code for your monkey

require Exporter;
require DynaLoader;
require AutoLoader;

use strict;
use Expect;
use Term::ReadKey; # for passwords
use IO::Socket::INET;

@coqm::ISA = qw(Exporter AutoLoader DynaLoader);

# functions to export
@coqm::EXPORT = qw(create_session, set_session);

my $dflt_timeout = 60;
my $rh_session;

sub set_debug
{
	my ($debug) = (@_);
	if ( $debug )
	{
		$Expect::Log_Stdout = 1;
	}
	else
	{
		$Expect::Log_Stdout = 0;
	}
}

sub get_password
{
	my ($prompt) = (@_);
	if ( ! $prompt ){ $prompt = "Password: "; }

	my $passwd;
        print "$prompt";
	ReadMode 'noecho';
	my $passwd = ReadLine(0);
        chomp($passwd);
        #$passwd = substr($passwd, 0, 15);
	ReadMode 'normal';
	print "\n";
	return "$passwd";
}

sub get_method
{
	my ($device) = (@_);
	my $method;
	my $socket = IO::Socket::INET->new("${device}:22");
	if ( $socket )
	{
		$method = "ssh";
		$socket->close();
	}
	else
	{
		$method = "telnet";
	}
	return $method;
}

sub create_session
{
	my ($method, $device, $username, $passwd, $debug, $timeout ) = (@_);
	
	# first just go ahead and set the debug flag
	set_debug($debug);
	
	if ( ! $method )
	{
		$method = get_method( $device );
	}
	my %session;
	$session{ 'device' } = $device;
	$session{ 'method' } = $method;
	$session{ 'username' } = $username;
	$session{ 'password' } = $passwd;
	$session{ 'timeout' } = $timeout;
	if ( $timeout == 0 ) { $session{ 'timeout' } = $dflt_timeout; }
	$rh_session = \%session;
	return $rh_session;

}

sub set_session
{
	my ($rh_sess) = (@_);
	$rh_session = $rh_sess;
}
	


sub login
{
	if( $$rh_session{ 'method' } eq "ssh" )
	{
		$$rh_session{ 'expect_handle' } =  Expect->spawn( "sh" ) ;
		my $h = $$rh_session{ 'expect_handle' };
		print $h "ssh -l $$rh_session{ 'username' } $$rh_session{ 'device' }\n";
	}
	else
	{
		$$rh_session{ 'expect_handle' } =  Expect->spawn( $$rh_session{ 'method' } , $$rh_session{ 'device' } );
	}
	my $eh = $$rh_session{ 'expect_handle' };
	my $timeout = $$rh_session{ 'timeout' };

	my $pwdsent;
	while ( ! $pwdsent )
	{
		my $match = $eh->expect( $timeout, 
					 "assword:",
					 "sername:",
					 "ogin:",
					 "Are you sure you want to continue connecting",
					 "ssh_exchange_identification: Connection closed by remote host",
					 "ssh_exchange_identification: read: Connection reset by peer"
					 );

		if ( !defined($match) )
		{
			return "didn't get a recognizable prompt in $timeout";
		}
		elsif( $match == 1 )
		{
			print $eh "$$rh_session{ 'password' }\r";
			$pwdsent = "yes";
		}
		elsif( $match == 2 || $match == 3 )
		{
			print $eh "$$rh_session{ 'username' }\r";
		}
		elsif( $match == 4 )
		{
			print $eh "yes\r";
		}
		elsif( $match == 5 || $match == 6 )
		{
			# if the other side has an ssh server
			# but it refuses our connection
			# fail over to telnet
			$eh->hard_close();
			$$rh_session{ 'method' } = "telnet";
			return login();
		}
		
	}
	my $match = $eh->expect( $timeout, 
				 "assword: ",
				 "sername: ",
				 "ogin: ",
				 "%",
				 ">",
				 " \(enable\) ", #catos enable
				 "#"
				);

	# turn off more
	# disable clipaging  # extremes
	# set length 0       # catos
	# terminal length 0  # ios

	if ( !defined($match)  || $match <= 4 )
	{
		return "error on login attempt (bad username/passwd?)";
	}
	else
	{
		# we're in, so get a prompt and get out!
		return get_prompt();
	}

}

sub logout
{
	my $eh = $$rh_session{ 'expect_handle' };
	my $timeout = $$rh_session{ 'timeout' };
	print $eh "exit\r";
	my $exited;
	while (! $exited)
	{
		my $match = $eh->expect( $timeout,
					"Connection closed",  # telnet
					"Connection to",  # ssh
					"Do you wish to save your configuration changes? (y/n)", #extremes, by default we say no
				);
		if ( !defined( $match ) )
		{
			last;
		}
		elsif( $match == 3 )
		{
			print $eh "n\r";
		}
		else
		{
			$exited = "yes";
			last;
		}
	}
	if ( $exited )
	{
		if ( $$rh_session{ 'method' } eq "ssh" )
		{
			print $eh "exit\r";
		}
		#$eh->soft_close();
		#sleep 1;
		$eh->hard_close();
		return 0;
	}
	else
	{
		#$eh->soft_close();
		#sleep 1;
		$eh->hard_close();
		return "failed to log out!";
	}

	
}

sub get_prompt
{
	my $eh = $$rh_session{ 'expect_handle' };
	my $timeout = $$rh_session{ 'timeout' };
	print $eh "\r";
	my @prompt = $eh->expect( $timeout,
					'-re', '^.*? \(enable\) ', # catos enable
					'-re','^.*?> ', #catos or hp
					'-re','^.*?>',  #ios
					'-re','^.*?# ', #extreme enable
					'-re','^.*?#',  #ios enable
					);
	# just in case the enable showed up but didn't match above
	if ($eh->exp_after() eq '(enable) ')
	{
		$prompt[2] .= $eh->exp_after();
	}
	if ( $prompt[2] =~ /^.+:[0-9]+ [#>] / )
	{
		#extreme's have an ever changing prompt, so strip out changing part
		#$prompt[2] =~ s/^(.+:)[0-9]+ [#>] /$1/g;
		$prompt[2] =~ s/^.+:[0-9]+//g;
	}
	if ( $prompt[0] )
	{
		$$rh_session{ 'prompt' } = $prompt[2];
		# immediately expect another prompt
		# just to get rid of any that might be lying around
		# in the buffer
		$eh->expect( 1, $prompt[2] );
		return 0;
	}
	else
	{
		return "get_prompt: failed to get a recognizeable prompt!";
	}
}

sub get_response
{
	my ($timeout) = (@_);
	my $eh = $$rh_session{ 'expect_handle' };
	my $prompt = $$rh_session{ 'prompt' };
	if( ! $timeout ) { $timeout = $$rh_session{ 'timeout' }; }

	my $output;

	my $match = 2;
	while ( $match > 1 )
	{
		$match = $eh->expect( $timeout, 
					$prompt,
					"More--",
					"Do you wish to continue y/n [n]?",
					'-re', '.*Press .SPACE. to cont.*'
					);
		$output = $output . "\n" . $eh->exp_before();
		if ( $match == 2 )
		{
			# more gets a space
			print $eh " ";
		}
		if ( $match == 3 )
		{
			# this guy gets a y
			print $eh "y\r";
		}
		if ( $match == 4 )
		{
			# eXtreme more gets a space
			print $eh " ";
		}
	}
	return $output;
}

sub run_command
{
	my ($command, $timeout) = (@_);
	my $eh = $$rh_session{ 'expect_handle' };
	print $eh $command . "\r";
	return get_response( $timeout );
	
}

sub run_multi_line_command
{
	my ($ra_lines, $delay, $timeout) = (@_);
	if ( ! $delay ){ $delay = 1; }
	my $eh = $$rh_session{ 'expect_handle' };
	foreach my $command (@$ra_lines)
	{
		print $eh $command . "\r";
		sleep $delay;
	}
	return get_response( $timeout );
}

sub get_type
{

	my $output = run_command( "show ver" ) . "\n";
        if($output =~  m/Extremeware +Version/)
        {
		 $$rh_session{'type'} = "extreme";
              if($output =~ m/Version\s((.*?))\)/)
              {
                
		$$rh_session{'version'} = "$1\)";
              }
        }
        elsif($output =~ m/[Cc]isco/ && $output =~ m/[Ss]ystems/ )
        {
               if($output =~ m/IOS/)
               {
                 $$rh_session{'type'} = "ios";
		 if($output =~ m/Version\s((.*?)),/)
                     {
                        $$rh_session{'version'} = "$1";
                     }
               }
                elsif($output =~ m/NmpSW\:\s((.*?)\))/)
                {
                        $$rh_session{'type'} = "catos";
			$$rh_session{'version'} = "$1";
                }
        }
        elsif($output =~ m/Force10 Networks/ )
	{
		$$rh_session{'type'} = "force10";
		 if($output =~ m/Force10 Application Software Version: ([0-9\.]+)/)
                     {
                        $$rh_session{'version'} = "$1";
                     }
	}
}

sub tftp_from_device
{
	my ($tftp_server, $src, $dest, $timeout, $options) = (@_);
	my $prompt = $$rh_session{ 'prompt' };
	my $eh = $$rh_session{ 'expect_handle' };
	if( ! $timeout ) { $timeout = $$rh_session{ 'timeout' }; }
	my $type = $$rh_session{ 'type' };
	if ( $type eq "ios" || $type eq "force10" )
	{
		print $eh "copy $src tftp://$tftp_server/$dest $options\r";
		
		my $match = 2;
		while ( $match > 1 )
		{
			$match = $eh->expect( $timeout,
						$prompt,
						"Address or name of remote host [$tftp_server]?",
						"estination filename [$dest]?",
						"%");
			if ( $match == 2)
			{
			 	print $eh "\r";
			}
			if ( $match == 3)
			{
				print $eh "\r";
			}
			if ( $match == 4)
			{
				return "error with tftp!\n"
			}
			if ( !defined( $match ) )
			{
			 	return "error with tftp! (timeout)\n";
			}
		}	
	}
	if ( $type eq "extreme" )
	{
		print $eh "upload $src $tftp_server $dest $options\r";
		my $output = get_response();
			if($output =~ m/rror/)
			{	
				return "error with tftp!\n";	
				#return $output;
			}
	}
	if ( $type eq "catos" )
	{
		print $eh "copy $src tftp:$dest $options\r";

		my $match = 2;
		while ( $match > 1 )
		{
			$match = $eh->expect( $timeout, "success", "ddress or name of remote host", "pload configuration", "error", $prompt );

			if ( !defined( $match ) )
			{
				return "error with tftp! (timeout)\r";
			}
			if ( $match == 1 )
			{
				$match = $eh->expect( $timeout, $prompt );
				if ( ! defined( $match ) )
				{
					return "failed to get a prompt back after a successful tftp";
				}
			}
			if ( $match == 2 )
			{
				print $eh "$tftp_server\r";
			}
			if ( $match == 3 )
			{
				print $eh "y\r"
			}
			if ( $match == 4 )
			{
				return "error with tftp!\n";
			}
			if ( $match == 5 )
			{
				return "got prompt back before getting the tftp OK message, tftp probably failed.";
			}
		}
	}
	if ( $type eq "force10" )
	{
		#ANL-TG-E1200#copy running-config tftp:
		#Address or name of remote host []: 140.221.9.24
		#Destination file name [startup-config]: f1.conf
		#!!
		#37916 bytes successfully copied
	}
}

sub tftp_from_configuration
{
	my ($tftp_server, $dest, $timeout) = (@_);
	my $type = $$rh_session{ 'type' };
	if ( $type eq "ios" || $type eq "force10" )
	{
		return tftp_from_device( $tftp_server, "running-config", $dest, $timeout );
	}
	if ( $type eq "extreme" )
	{
		return tftp_from_device( $tftp_server, "configuration", $dest, $timeout );
	}
	if ( $type eq "catos" )
	{
		my $return = tftp_from_device( $tftp_server, "config", $dest, $timeout,  "all" );
		if ( $return )
		{
			# some old catos boxes require that you do it this way
			return catos_write_network( $tftp_server, $dest, $timeout );
		}
	}
}

sub catos_write_network
{
	my ($tftp_server, $dest, $timeout ) = (@_);
	
	my $eh = $$rh_session{ 'expect_handle' };
	my $prompt = $$rh_session{ 'prompt' };
	if ( ! $timeout ) { $timeout = $$rh_session{ 'timeout' }; }

	# some old school catos boxes do it this way
	# > write network all
	# IP address or name of remote host? [zero] 140.221.9.24
	# Name of configuration file?[myswitch.cfg] dream.conf
	# Upload configuration to dream.conf on 140.221.9.24 (y/n) [n]? y
	# ....
	# Finished network upload.  (42112 bytes)
	# >
	print $eh "write network all\r";
	my $match = 2;
	while ( $match > 1 )
	{
		$match = $eh->expect( $timeout, 
						"Finished network upload",
						"IP address or name of remote host?",
						"Name of configuration file?",
						-re, "Upload configuration.*.y.n. .n.\?",
						"TFTP: Abnormal Termination",
						$prompt
						);

		if ( ! defined( $match ) )
		{
			return "didn't get a response within $timeout seconds while attempting tftp"
		}
		if ( $match == 1 )
		{
			$match = $eh->expect( $timeout, $prompt );
			if ( ! defined( $match ) )
			{
				return "failed to get a prompt back after a successful tftp";
			}
		}
		if ( $match == 2 )
		{
			print $eh "$tftp_server\r";
		}
		if ( $match == 3 )
		{
			print $eh "$dest\r";
		}
		if ( $match == 4 )
		{
			print $eh "y\r";
		}
		if ( $match == 5 )
		{
			return "error with tftp!\n";
		}
		if ( $match == 6 )
		{
			return "got prompt back before getting the tftp OK message, tftp probably failed.";
		}
	}				
}


sub tftp_to_device
{
	my ($tftp_server, $src, $dest, $timeout, $option ) = (@_);
	my $eh = $$rh_session{ 'expect_handle' };
	if ( !$timeout )
	{
		$timeout = $$rh_session{ 'timeout' };
	}
	my $prompt = $$rh_session{ 'prompt' };
	my $type = $$rh_session{ 'type' };

	if ( $type eq "ios" || $type eq "force10" )
	{
		print $eh "copy tftp://$tftp_server/$src $dest\r";
		my $match = 2;
		while ($match > 1)
		{
			$match = $eh->expect( $timeout,
							"[OK",
							"]? ",
							"%Warning:There is a file already existing with this name",
							"Do you want to over write?",
							"%",
							$prompt
							);
			if ( ! defined( $match ) )
			{
				return "didn't get a response within $timeout seconds while attempting tftp"
			}
			if ( $match == 1 )
			{
				$match = $eh->expect( $timeout, $prompt );
				if ( ! defined( $match ) )
				{
					return "failed to get a prompt back after a successful tftp";
				}
			}
			if ( $match == 2)
			{
				print $eh "\r";
			}
			if ( $match == 3 )
			{
				# nothing to do, this match is just here so the more
				# general error match isn't triggered
			}
			if ( $match == 4 )
			{
				print $eh "\r";
			}
			if ( $match == 5 )
			{
			        return "errror with tftp!";  
			}
			if ( $match == 6 )
			{
				if ( $type eq "force10" )
				{
					# expect behavior on a force10
					return 0;
				}
				else
				{
					# not on a cisco
					return "got prompt back before getting the tftp OK message, tftp probably failed.";
				}
		        }
		}
		return 0;
	}
	if ( $type eq "extreme" )
	{
		#if ( $dest = "configuration" ) { $option = "incremental" ; }
		#if ( $option ne "incremental" ) { return 0; }
		print $eh "download $dest $tftp_server $src $option\r";
		my $output = get_response();
		if ( $output =~ /^Error/ )
		{
			return "error while tftping";
		}
		return 0;
	}
	if ( $type eq "catos" )
	{
		print $eh "copy tftp://$src $dest\r";
		my $match = 2;
		while ($match > 1)
		{
			$match = $eh->expect( $timeout,
							"Finished network download.",
							'-re', 'IP address or name of remote host \[.*\]\? ',
							"Configure using tftp://$src (y/n) [n]? ",
							"bytes available on device bootflash, proceed (y/n) [n]?",
							$prompt
							);
			if ( ! defined( $match ) )
			{
				return "didn't get a response within $timeout seconds while attempting tftp"
			}
			if ( $match == 1 )
			{
				$match = $eh->expect( $timeout, $prompt );
				if ( ! defined( $match ) )
				{
					return "failed to get a prompt back after a successful tftp";
				}
			}
			if ( $match == 2 )
			{
				print $eh "$tftp_server\r";
			}
			if ( $match == 3 )
			{
				print $eh "y\r";
			}
			if ( $match == 4 )
			{
				print $eh "y\r";
			}
			if ( $match == 5 )
			{
				return "got prompt back before getting the tftp OK message, tftp probably failed.";
			}
		}
		return 0;
	}


}

sub tftp_to_configuration
{
	my ($tftp_server, $src, $timeout ) = (@_);
	my $type = $$rh_session{ 'type' };
	if ( $type eq "ios" || $type eq "force10" )
	{
		return tftp_to_device( $tftp_server, $src, "running-config", $timeout );
	}
	if ( $type eq "extreme" )
	{
		return tftp_to_device( $tftp_server, $src, "configuration", $timeout, "incremental" );
	}
	if ( $type eq "catos" )
	{
		my $return = tftp_to_device( $tftp_server, $src, "conf", $timeout );
		if ( $return )
		{
			return catos_configure_network( $tftp_server, $src, $timeout );
		}
	}
}

sub catos_configure_network
{
	my ($tftp_server, $src, $timeout ) = (@_);
	
	my $eh = $$rh_session{ 'expect_handle' };
	my $prompt = $$rh_session{ 'prompt' };
	if ( ! $timeout ) { $timeout = $$rh_session{ 'timeout' }; }

	# some old school catos boxes do it this way
	# > configure network
	# IP address or name of remote host [140.221.9.24]? 
	# Name of file to copy from [dream.conf]? snmp.catos
	# Configure using snmp.catos from 140.221.9.24 (y/n) [n]? y
	# ....
	# Finished network download.  (42112 bytes)
	# >
	print $eh "configure network\r";
	my $match = 2;
	while ( $match > 1 )
	{
		$match = $eh->expect( $timeout, 
						"Finished network download",
						"IP address or name of remote host",
						"Name of file to copy from",
						"Configure using $src from $tftp_server (y/n) [n]?",
						"TFTP: Abnormal Termination",
						$prompt
						);

		if ( ! defined( $match ) )
		{
			return "didn't get a response within $timeout seconds while attempting tftp"
		}
		if ( $match == 1 )
		{
			$match = $eh->expect( $timeout, $prompt );
			if ( ! defined( $match ) )
			{
				return "failed to get a prompt back after a successful tftp";
			}
		}
		if ( $match == 2 )
		{
			print $eh "$tftp_server\r";
		}
		if ( $match == 3 )
		{
			print $eh "$src\r";
		}
		if ( $match == 4 )
		{
			print $eh "y\r";
		}
		if ( $match == 5 )
		{
			return "error with tftp!\n";
		}
		if ( $match == 6 )
		{
			return "got prompt back before getting the tftp OK message, tftp probably failed.";
		}
	}				
}

sub save_configuration
{
	my $eh = $$rh_session{ 'expect_handle' };
	my $timeout = $$rh_session{ 'timeout' };
	my $prompt = $$rh_session{ 'prompt' };
	my $type = $$rh_session{ 'type' };

	if ( $type eq "catos" )
	{
		return 0; # with CatOS you always win!
	}
	if ( $type eq "ios"  || $type eq "force10" )
	{
		print $eh "copy running-config startup-config\r";
		my $match = 2;
		while ($match > 1)
		{
			$match = $eh->expect( $timeout,
							$prompt,
							"]? ",
							"%",
							"Proceed to copy the file [confirm yes/no]:",
							);
			if ( ! defined( $match ) )
			{
				return "didn't get a response within $timeout seconds while attempting to save the configuration"
			}
			if ( $match == 2 )
			{
				print $eh "\r";
			}
			if ( $match == 3 )
			{
				return "error while attempting to save config";
			}
			if ( $match == 4 )
			{
				print $eh "yes\r";
			}
		}
		return 0;
	}
	if ( $type eq "extreme" )
	{
		print $eh "save configuration\r";
		my $match = 2;
		while ($match > 1)
		{
			$match = $eh->expect( $timeout,
							$prompt,
							"Do you want to save to the primary configuration database?",
							"Error",
							);
			if ( ! defined( $match ) )
			{
				return "didn't get a response within $timeout seconds while attempting to save the configuration"
			}
			if ( $match == 2 )
			{
				print $eh "y\r";
			}
			if ( $match == 3 )
			{
				return "error while attempting to save config";
			}
		}
	}
}

1;


__END__

=head1 NAME

nde - module for interacting with network devices using Expect.pm

=head1 SYNOPSIS

	use nde;
	my $passwd = nde::get_password();
	my $rh = nde::create_session( $method, $device, $username, $passwd, $debug, $timeout  );
	my $err = nde::login();
	my $output = nde::run_command( $command );
	$err = nde::logout();



=head1 DESCRIPTION

The nde (or network device expect) module is designed to provide a consistent interface for logging into and running commands on network devices of all types.
Currently the support platforms are Cisco CatOS, Cisco IOS, ExtremeOS, and Force10.


=head1 FUNCTIONS

=over 4

=item *
create_session($method, $device, $username, $passwd, $debug, $timeout ) 

Info:
  Creates a hash out of the information passed, and returns a ref to that hash.
  Also points nde.pm's internal session hash reference to this new hash.
  This info is then used for all subsequent nde function calls (ie, login(), etc).
  Moreover, this hash will have additional keys added to it as other functions (such as login() and get_type()) are called.
  Use set_session() to re-point nde.pm's internal session hash reference.

Argument(s):

  $method (optional)

    Currently create_session accepts two 'methods' for connecting to a network device: 'ssh' and 'telnet'
    Both commands should be in the user's PATH.
    IF no method is passed, the get_method() is called to try and figure out which method to use.

  $device

    Network device to connect to, should be an IP Address or valid DNS Hostname.

  $username

    username to use when logging onto device.

  $passwd

    password to use when logging onto device.

  $debug (optional)

    If true, then interactive session will be echoed to user's terminal as it takes place.  Additional debug info maybe displayed as well.

  $timeout (optional)

    How long (in seconds) to wait for output from any given response from the device.  Default value is 60 seconds.


Return value:

  returns a reference to an nde session hash.

=item *
set_session($rh_sess) 

Info:
   Points nde.pm's internal session hash reference to passed value.
   The info in the referenced hash will then be used for all subsequent nde function calls (ie, login(), etc).
   The general idea is that by using create_session and set_session in proper concert, one can use nde to connect to and run commands on multiple devices simultaneously.

Argument(s):

  $rh_sess

    Reference to a hash of the type returned by create_session().


Return value:

  void

=item *
set_debug($debug) 

Info:
  Internal helper function which can be used to turn on debugging output.  If called with a true value, all future input and output for an expect session will be printed to the terminal.  Allows one to see an expect session as it happens.  Very useful for debugging code.

Argument(s):

  $debug

     If set to a true value (any string or non-zero number), function will turn debug output on.
     If set to a false value (0 or ""), function will turn debug output off.


Return value:

  void

=item *
get_password($prompt) 

Info:
  Helper function.  Uses the Term::ReadKey module to turn off kyeboard echoing and reads a line off standard in.  It then turns keyboard echo back on and returns the line it read in.
  Basically, this is just a fucntion to read in a user's password w/o having it appear on the screen as he/she types it.  It's included in this module since nearly every program that tries to log onto a network device needs such a routine.

Argument(s):

  $prompt (optional)

    By default the function prints the prompt "Password: " to the screen.  Pass in another text string to have it print that instead.


Return value:

  returns a scalar whose value comes from reading a single line from <STDIN>.  For convience, chomp() is called on this line.

=item *
get_method($device) 

Info:
  An internal function to determine how to log onto a device.
  Uses the perl module IO::Socket::INET to attempt to open a connection to device on port 22.  If the attempt succeeds it is assumed that ssh should be used to connect to the device.   If the attempt fails it is assumed that telnet should be used to connect to the device.

Argument(s):

  $device

    Valid hostname or IP address of the device to check for SSH or Telnet protocol support.


Return value:

  returns a scalar whose value is either "ssh" or "telnet" indicating that one or the other protocol is supported by the device.  If the device supports both protocols "ssh" is returned.

=item *
login()

Info:
  Uses Expect.pm, as well as the info setup via create_session() to log onto a network device.
  Adds two values to the session hash:
    'prompt' = string containing the device's prompt (ie 'switch>', etc)
    'expect_handle' = Expect.pm object, useful for directly interacting with device via Expect.pm's methods.

Return value:

  0 on success
  error string on failure

=item *
logout()

Info:
  Attempts to gracefully close connection to device.

Return value:

  0 on success
  error string on failure

=item *
get_type()

Info:
  Attempts to determine what type of equipment a device is.
  Once this value is determined it is entered into nde's global session hash under the key 'type'.
  This value is used and required by most other nde functions to determine how to perform various functions (saving configuration, copying configuration to a tftp server, etc) on the connected device.
  Additionally, the function attempts to determine what version of code the device is running, this value is entered into the global session hash under the key 'version'.


Return value:

  0 on success
  error string on failure

=item *
get_prompt()

Info:
  Internal function used to determine what the standard 'prompt' presented by a device is.
  This function is called by login(), so there should be no reason to call it seperately.
  It inserts the value if finds for the prompt into the current nde session hash under the key 'prompt'

Return value:

  0 on success
  error string on failure

=item *
get_response($timeout) 

Info:
  Collects all the output from a device up until the next prompt.
  Handles 'more', 'continue' and other screen paging prompts.
  In general, this function is provided as a quick way to collect a device's response after a command has been sent to it.
has been

Argument(s):

  $timeout (optional)

    How long to wait for a prompt to show up.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  returns a string which is the composite of all output from a device up until the next prompt.

=item *
run_command($command, $timeout) 

Info:
  Sends the passed command to the currently active nde session (as set by the last call to create_session() or set_session()), collects and returns any output from the command.
  No account is made for any extra questions or information the command may prompt the user for.  The command is assumed to be of the type that simply spits out some output and drops the user back to a prompt, ie commands like "show int", "show proc cpu", etc.

Argument(s):

  $command

    command to run on device.

  $timeout (optional)

    How long to wait for a prompt to show up after running $command.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  returns string which is the output from running $command

=item *
run_multi_line_command($ra_lines, $delay, $timeout) 

Info:
  A slight tweak on run_command().  This function is meant to handle commands which prompt a user for further parameters/information.
  The function takes an array of lines, it then cycles through this array sending each line and then waiting $delay seconds.  After all the lines have been sent, the function calls get_response() to collect all the output up until the next prompt.
  So the idea isn't to run multiple commands, but rather to run one command which requires multiple lines of input to execute.

Argument(s):

  $ra_lines

    reference to an array of lines to send to device before waiting for a prompt.

  $delay (optional)

    how long to wait between sending each line of $ra_lines.  If no value is given, the function will wait 1 seond between each sending each line.

  $timeout (optional)

    How long to wait for a prompt to show up after sending lines from $ra_command.  If no value is passed, the 'timeout' value for the current nde session hash is used.



Return value:

  returns a string which is the collected output from the sending of the first line in $ra_commands to the next prompt.


=item *
tftp_from_device($tftp_server, $src, $dest, $timeout, $options) 

Info:
  Attempts a tftp transfer of a file ($src) from a device.
  What exactly constitues a 'file' depends on what type of device one is connected to.  On Force10, Cisco IOS and Cisco CatOS devices $src can be any valid argument for the 'copy' command (ie 'running-config', 'flash:vlan.dat', etc).  On ExtremeOS devices $src can be any valid argument to the 'upload' command (ie. 'configuration' or 'system-dump').

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to upload file to.

  $src

    source 'file' to transfer from device.  (see Info)

  $dest

    destination file on the tftp server to copy device's configuration to.  Function makes no effort to verify that $dest is a valid destination filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world write) to be written to via tftp.

  $timeout (optional)

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.


  $options (optional)

    A string which is appended to the end of the 'copy' or 'upload' command.
    Provides a way to specify the 'all' option to the 'copy config' command, which required when retrieving configuration files from CatOS devices.


Return value:

  0 on success
  error string on failure

=item *
tftp_from_configuration($tftp_server, $dest, $timeout) 

Info:
  A wrapper function which will copy a device's current running config to the specified tftp server.  Basically, it knows the right flags to call tftp_from_device() with for all the supported device types in order to copy the device's running config (as opposed to an arbitray file) to the tftp server.

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to upload file to.

  $dest (optional)

    destination file on the tftp server to copy device's configuration to.  Function makes no effort to verify that $dest is a valid destination filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world write) to be written to via tftp.

  $timeout

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  0 on success
  error string on failure

=item *
catos_write_network($tftp_server, $dest, $timeout ) 

Info:
  Older Cisco CatOS devices use the command 'write network all' to transfer their configuration via tftp.  This is a function to deal with these devices.  It is called by tftp_from_configuration() as needed.

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to upload file to.

  $dest

    destination file on the tftp server to copy device's configuration to.  Function makes no effort to verify that $dest is a valid destination filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world write) to be written to via tftp.

  $timeout (optional)

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  0 on success
  error string on failure

=item *
tftp_to_device($tftp_server, $src, $dest, $timeout, $option ) 

Info:
  Attempts a tftp transfer of a file ($src) to a device.
  Where the file can end up ($dest) depends on what type of device one is connected to.  On Force10, Cisco IOS and Cisco CatOS devices $dest can be any valid argument for the 'copy' command (ie 'running-config', 'flash:ios.image', etc).  On ExtremeOS devices $dest can be any valid argument to the 'download' command (ie. 'configuration' or 'system').

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to download file from.

  $src

    source file on the tftp server to copy to the device.  Function makes no effort to verify that $src is a valid source filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world read) to be read from via tftp.


  $dest

    destination file on the device.

  $timeout (optional)

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.

  $option (optional)

    A string which is appended to the end of the 'copy' or 'download' command.
    Provides a way to specify the 'incremental' option to the 'download config' command on ExtremeOS devices.

Return value:

  0 on success
  error string on failure

=item *
tftp_to_configuration($tftp_server, $src, $timeout ) 

Info:
  A wrapper function which will copy a file from the specified tftp server to a device's current running config.  Basically, it knows the right flags to call tftp_to_device() with for all the supported device types in order to copy a file into a device's running config.  For all device types, this copy is 'incremental' in nature, that is it adds the contents of $src to the device's running config, rather than replacing that config wholesale with $src.
  

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to download file from.

  $src

    source file on the tftp server to copy to the device.  Function makes no effort to verify that $src is a valid source filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world read) to be read from via tftp.

  $timeout (optional)

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  0 on success
  error string on failure

=item *
catos_configure_network($tftp_server, $src, $timeout ) 

Info:
  Older Cisco CatOS devices use the command 'conifigure network' to transfer configs from a tftp server to their configuration.  This is a function to deal with these devices.  It is called by tftp_to_configuration() as needed.

Argument(s):

  $tftp_server

    IP address or valid DNS hostname of the tftp server to download file from.

  $src (optional)

    source file on the tftp server to copy to the device.  Function makes no effort to verify that $src is a valid source filename, rather it  assumes that this file exists on the tftp server and has proper permissions (world read) to be read from via tftp.

  $timeout (optional)

    How long to wait for a prompt to show up after the initiating tftp transfer.  If no value is passed, the 'timeout' value for the current nde session hash is used.


Return value:

  0 on success
  error string on failure

=item *
save_configuration()

Info:
  Attempts to save the configuration on a device.  On different device types this means different things.  On Force10 and Cisco IOS it means running "copy run start", on ExtremeOS it means running "save configuration",  on Cisco CatOS it means doing nothing.

Return value:

  returns 0 on succes
  returns an error string on failure


=back

